﻿/*********************************************************
 * 开发人员：小兵
 * 创建时间：2012-10-9 10:10:47
 * 描述说明：
 * 
 * 更改历史：
 * 
 * *******************************************************/
namespace Mozlite.Data.MySql
{
    using System;
    using System.Web;
    using System.Data;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Mozlite.Data.Components.Installer;
    using System.Text.RegularExpressions;

    /// <summary>
    /// MYSQL数据库安装类。
    /// </summary>
    public class MySqlInstaller : BaseInstaller<MySqlHelper>
    {
        private const DatabaseVersion LatestVersionSupported = DatabaseVersion.Version1_1;

        private static readonly VersionSpecification[] versionSpecs = new VersionSpecification[] {
                    new VersionSpecification("SELECT version()", DatabaseVersion.None) };

        /// <summary>
        /// 判断当前数据库是否为可更新数据库。
        /// </summary>
        /// <value>
        /// 如果可以更新数据库返回<c>true</c>，否则返回<c>false</c>。
        /// </value>
        /// <remarks>空数据库不能更新，只能安装。</remarks>
        public override bool CanUpgrade
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// 版本唯一识别辅助方法。
        /// </summary>
        protected override VersionSpecification[] VersionSpecs
        {
            get { return versionSpecs; }
        }
        /// <summary>
        /// 初始化类<see cref="MySqlInstaller"/>。
        /// </summary>
        /// <param name="sqlHelper">数据库辅助接口实例对象。</param>
        public MySqlInstaller(MySqlHelper sqlHelper)
            : base(sqlHelper, LatestVersionSupported)
        { }

        /// <summary>
        /// 获取全新安装脚本字符串。
        /// </summary>
        protected override string FullInstallSql
        {
            get { return GetMySqlVersion() >= 5 ? SqlResources.Full : ConvertToMySql4(SqlResources.Full); }
        }

        /// <summary>
        /// 获取更新脚本字符串。
        /// </summary>
        protected override string UpgradeSql
        {
            get
            {
                string upgradeFile = string.Format("{0}_Upgrade", CurrentVersion.ToString());
                var sql = SqlResources.ResourceManager.GetString(upgradeFile);
                return GetMySqlVersion() >= 5 ? sql : ConvertToMySql4(sql);
            }
        }

        /// <summary>
        /// 将SQL语句转换为MySQL 4.x版本。
        /// </summary>
        /// <param name="sql">MySQL 5+版本以上的SQL语句。</param>
        /// <returns>返回合法的SQL语句。</returns>
        protected virtual string ConvertToMySql4(string sql)
        {
            return Regex.Replace(sql, @"(^?[ | n]varchar \(([2-9][5-9][6-9]|[0-9]{4,5})\)*)",
                                      @" text", RegexOptions.IgnoreCase);
        }

        /// <summary>
        /// 获取数据库主要版本。
        /// </summary>
        /// <returns>返回数据库主要版本。</returns>
        protected virtual int GetMySqlVersion()
        {
            return int.Parse(SqlHelper.ExecuteScalar<string>("SELECT SUBSTRING(VERSION(),1,1)"));
        }
    }
}
