﻿/***************************************************************
 * 开发人员：网魂小兵@http://www.mozlite.com
 * 
 * 缓存提供者接口。
 * 
 * 最后修改：2011-10-28
 * ************************************************************/
namespace Mozlite.Caching
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Collections;
    using System.Web.Caching;
    /// <summary>
    /// 缓存提供者接口。
    /// </summary>
    public interface ICacheProvider
    {
        /// <summary>
        /// 清理所有缓存。
        /// </summary>
        void Clear();
        /// <summary>
        /// 获取缓存键中的所有缓存列表。
        /// </summary>
        /// <typeparam name="T">缓存对象类型。</typeparam>
        /// <param name="keys">缓存键迭代器实例对象。</param>
        /// <returns>返回缓存列表。</returns>
        IDictionary<string, T> Get<T>(IEnumerable<string> keys);
        /// <summary>
        /// 获取当前键的缓存对象。
        /// </summary>
        /// <typeparam name="T">缓存类型。</typeparam>
        /// <param name="key">缓存键。</param>
        /// <returns>返回缓存对象。</returns>
        T Get<T>(string key);
        /// <summary>
        /// 获取当前键的缓存对象。
        /// </summary>
        /// <param name="key">缓存键。</param>
        /// <returns>返回缓存对象。</returns>
        object Get(string key);
        /// <summary>
        /// 获取枚举非泛型字典实例对象。
        /// </summary>
        /// <returns>返回枚举非泛型字典实例对象。</returns>
        IDictionaryEnumerator GetEnumerator();
        /// <summary>
        /// 将所有字典中的缓存项插入到缓存中。
        /// </summary>
        /// <typeparam name="T">缓存类型。</typeparam>
        /// <param name="items">缓存列表实例对象。</param>
        /// <param name="dependencies">缓存依赖项。</param>
        /// <param name="absoluteExpiration">缓存过期绝对日期。</param>
        /// <param name="slidingExpiration">缓存过期相对日期。</param>
        /// <param name="priority">缓存优先级。</param>
        /// <param name="onRemoveCallback">单移除缓存触发的回调。</param>
        void Insert<T>(IDictionary<string, T> items, CacheDependency dependencies, DateTime absoluteExpiration, TimeSpan slidingExpiration, CacheItemPriority priority, CacheItemRemovedCallback onRemoveCallback);
        /// <summary>
        /// 将对象插入到缓存中。
        /// </summary>
        /// <param name="key">缓存键。</param>
        /// <param name="value">缓存对象。</param>
        /// <param name="dependencies">缓存依赖项。</param>
        /// <param name="absoluteExpiration">缓存过期绝对日期。</param>
        /// <param name="slidingExpiration">缓存过期相对日期。</param>
        /// <param name="priority">缓存优先级。</param>
        /// <param name="onRemoveCallback">单移除缓存触发的回调。</param>
        void Insert(string key, object value, CacheDependency dependencies, DateTime absoluteExpiration, TimeSpan slidingExpiration, CacheItemPriority priority, CacheItemRemovedCallback onRemoveCallback);
        /// <summary>
        /// 移除缓存。
        /// </summary>
        /// <param name="key">缓存键。</param>
        void Remove(string key);
        /// <summary>
        /// 获取缓存提供者名称。
        /// </summary>
        string ProviderName { get; }
    }
}
